import java.awt.Color;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import com.threed.jpct.Camera;
import com.threed.jpct.FrameBuffer;
import com.threed.jpct.IRenderer;
import com.threed.jpct.Lights;
import com.threed.jpct.Object3D;
import com.threed.jpct.Primitives;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.World;

/*
 * author: Ash
 * 
 * Created on 30.11.2003
 * 
 * This is the first part of the jPCT Tutorial. Here you will learn the basics of creating 
 * applications with jPCT. For the beginning you can see that every application consists 
 * of three parts: the initializing of the world, the creation of a container for our 
 * program and the main loop. Later we will extend our simple recipe.
 * 
 * This tutorial creates just a cube positioned at the origin of the coord system. In 
 * this tutorial we will use the software renderer so you don't need hardware acceleration 
 * to see the reasults.
 */

/*
 * Our main class starts here
 */
public class JPCTTutorial1 {

	
	// Some jPCT related lines. 
	private Object3D cube = null;
	private World theWorld = null;
	private Camera camera = null;
	private FrameBuffer buffer = null;
		
	/* 
	 * The width and the height of our framebuffer. Remember, this is _not_ the dimension
	 * of our Frame.
	 */	
	private int width = 640;
	private int height = 480;
	
	// Some lines for declaring AWT stuff
	private Frame mainFrame = null;
	private Graphics frameGraphics = null;
	private int titleBarHeight = 0;
	private int leftBorderWidth = 0;
	
	private boolean exit = false;
	
	// The constructor
	public JPCTTutorial1() {
		// First we create our empty world.
		theWorld = new World();

		/*
		 * Deactivate the overbright lighting, so our software renderer looks similar
		 * to the OpenGL renderer.	
		 */
		theWorld.getLights().setOverbrightLighting(Lights.OVERBRIGHT_LIGHTING_DISABLED);
		
		/*
		 * We want our scene to be lit by an ambient light source. With the parameters
		 * we control the intesity of the light so our positive params will light up
		 * the scene. With high values we can see our cube without any other light sources,
		 * but this won't look very realistic because every face reflects exactly the
		 * same amount of light so we won't see any edges.
		 */
		theWorld.setAmbientLight(20, 20, 20);
		
		/*
		 * Create a light and add it to our world. The first param ist the position of
		 * the light, the second one the color.
		 */		
	 	theWorld.addLight(new SimpleVector(10.0f, -10.0f, -30.0f), Color.red);
		
		// Here we create our cube with a scaling of 10 and add it to our world.				
		cube = Primitives.getCube(10.0f);
		theWorld.addObject(cube);
		
		/* 
		 * Get the camera from our world and move it 60 units "backwards" (in negative
		 * z-direction) so that we can see our cube.
		 */ 
		camera = theWorld.getCamera();
		camera.moveCamera(Camera.CAMERA_MOVEOUT, 60);
		
		// Now we have to build our objects.
		theWorld.buildAllObjects();
		
		// Create a frame.
		initializeFrame();
		
		// Start our main loop.
		gameloop();
	}
	
	// Here we create our frame and get its graphics context.
	private void initializeFrame() {
		// Create the frame, give it a title and pack everything.
		mainFrame = new Frame();
		mainFrame.setTitle("jPCT Tutorial #1: Setting up JPCT (Software)");
		mainFrame.pack();
		
		/* 
		 * Set the size of our frame. The size is calculated from the width and the height
		 * of the framebuffer and the frames borderwidth and -height (incl. title).
		 */		
		Insets insets = mainFrame.getInsets();
		titleBarHeight=insets.top;
		leftBorderWidth=insets.left;
		mainFrame.setSize(width+leftBorderWidth+insets.right, height+titleBarHeight+insets.bottom);
		
		// We don't want our frame to be resizeable.
		mainFrame.setResizable(false);
		
		// Show the frame.
		mainFrame.show();
		
		// Get our graphics context so that we can work with it.	
		frameGraphics = mainFrame.getGraphics();
		
		// Add a window listener (for closing)
		mainFrame.addWindowListener(new WindowEvents());	
	}			
	
	// The main loop of the application.
	private void gameloop() {
		/*
		 * Create our framebuffer with the given width and height. We don't 
		 * want to use a special sampling mode (we don't want to seperate our buffer
		 * grid into smaller units; no up- or downsampling). 
		 */
		buffer = new FrameBuffer(width, height, FrameBuffer.SAMPLINGMODE_NORMAL);
		
		// Use the software renderer.
		buffer.enableRenderer(IRenderer.RENDERER_SOFTWARE);
		
		// Optimize the buffer.
		buffer.optimizeBufferAccess();
		
		/* 
		 * The main loop. Calculate the scene, write it to the buffer, swap the buffers 
		 * and draw the content of current buffer onto the frame using the graphics context.
		 */ 
		while (!exit) {
			buffer.clear();
			theWorld.renderScene(buffer);
			theWorld.draw(buffer);
			buffer.update();
			buffer.display(frameGraphics, leftBorderWidth, titleBarHeight);
		}
		System.exit(0);
	}
	
	// Very complex stuff:)
	public static void main(String[] args) {
		JPCTTutorial1 helloWorld = new JPCTTutorial1();
	}
	
	// The listener. Just exit the application when i close the window.
	private class WindowEvents extends WindowAdapter {
		public void windowClosing(WindowEvent we) {
			exit = true;
		}
	}
}
