import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Point;

import com.threed.jpct.Camera;
import com.threed.jpct.FrameBuffer;
import com.threed.jpct.IRenderer;
import com.threed.jpct.Lights;
import com.threed.jpct.Object3D;
import com.threed.jpct.Primitives;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.World;


/*
 * author: Ash
 * 
 * Created on 08.12.2003
 * 
 * This tutorial will show you one way to change the renderer during runtime. The default
 * renderer is software, with a click on the OpenGL-button in the ChangeRenderer frame
 * you can switch to the hardware renderer.
 * 
 * Hint: For LWJGL use -Djava.library.path=d:/jpctapi/lib/lwjgl-0.7/ as vm argument 
 */
public class JPCTTutorial3 {
	
	private Object3D cube = null;
	private World theWorld = null;
	private Camera camera = null;
	private FrameBuffer buffer = null;
		
	private static int width = 640;
	private static int height = 480;
	
	private static Frame mainFrame = null;
	private Graphics frameGraphics = null;
	private static int titleBarHeight = 0;
	private static int leftBorderWidth = 0;
	
	private static boolean exit = false;
	
	// Use the software renderer?
	private static boolean softRenderer = true;
	
	// Change the renderer? (executed during main loop)
	private static boolean switchRenderer = false;
	
	// A Window showing different settings (will be extended in future tutorials)	
	private JPCTTutorial3Frame changeFrame = null;
		
	public JPCTTutorial3() {
		theWorld = new World();
						
		theWorld.getLights().setOverbrightLighting(Lights.OVERBRIGHT_LIGHTING_DISABLED);
		
		theWorld.setAmbientLight(20, 20, 20);
		
		theWorld.addLight(new SimpleVector(10.0f, -10.0f, -30.0f), Color.red);
		
		cube = Primitives.getCube(10.0f);
		theWorld.addObject(cube);
		
		camera = theWorld.getCamera();
		camera.moveCamera(Camera.CAMERA_MOVEOUT, 60);
		
		theWorld.buildAllObjects();
		
		initializeFrame();
		
		// Create our settings frame
		changeFrame = new JPCTTutorial3Frame();
		mainFrame.requestFocus();
				
		gameloop();
	}
	
	private void initializeFrame() {
		mainFrame = new Frame();
		mainFrame.setTitle("jPCT Tutorial #3: Change the renderer");
		mainFrame.pack();
		
		Insets insets = mainFrame.getInsets();
		titleBarHeight=insets.top;
		leftBorderWidth=insets.left;
		mainFrame.setSize(width+leftBorderWidth+insets.right, height+titleBarHeight+insets.bottom);
		mainFrame.setResizable(false);
		mainFrame.show();
		
		frameGraphics = mainFrame.getGraphics();
	}			
	
	private void gameloop() {
		
		// By default we are using the software renderer
		buffer = new FrameBuffer(width, height, FrameBuffer.SAMPLINGMODE_NORMAL);
		buffer.enableRenderer(IRenderer.RENDERER_SOFTWARE, IRenderer.MODE_OPENGL);		
		
		buffer.optimizeBufferAccess();
		
		while (!exit) {
					
			// Switch the renderer?
			if (switchRenderer)
				changeRenderer();
											
			buffer.clear();
			theWorld.renderScene(buffer);
			theWorld.draw(buffer);
			buffer.update();
			
			/* 
			 * OpenGL uses a different way to show the buffer, so we have to find
			 * out which renderer we are currently using and select the associated 
			 * method
			 */
			if (buffer.usesRenderer(IRenderer.RENDERER_OPENGL))
				buffer.displayGLOnly();
			else
				buffer.display(frameGraphics, leftBorderWidth, titleBarHeight);
		}
		System.exit(0);
	}
	
	/*
	 * This method changes our renderer; the request for changing the renderer is
	 * "queued" (with the variable switchRenderer) and executed in the main loop
	 */ 
	public void changeRenderer() {
		
		/*
		 * The softRenderer variable shows which renderer we want to use/we are
		 * currently using
		 * true - we want to use the software renderer
		 * false - we want to use the hardware renderer
		 */
		if (!softRenderer) {
			
			// Hide and dispose our AWT frame
			mainFrame.hide();
			mainFrame.dispose();
			
			// Enable the hardware renderer and disable software renderer
			buffer.enableRenderer(IRenderer.RENDERER_OPENGL, IRenderer.MODE_OPENGL);
			buffer.disableRenderer(IRenderer.RENDERER_SOFTWARE);
			
			// We are using the hardware renderer 
			softRenderer = false;
			
			/*
			 * With the code above we already changed our renderer and we won't do it again
			 * until the switchRenderer variable has changed again
			 */
			switchRenderer = false;
		}
		else {
			
			// Disable the hardware renderer and enable software renderer
			buffer.disableRenderer(IRenderer.RENDERER_OPENGL);
			buffer.enableRenderer(IRenderer.RENDERER_SOFTWARE, IRenderer.MODE_OPENGL);
			
			// We are using the software renderer
			softRenderer = true;
			
			// Initialize our frame	
			initializeFrame();
			
			/*
			 * With the code above we already changed our renderer and we won't do it again
			 * until the switchRenderer variable has changed again
			 */
			switchRenderer = false;
		}	
	}
	
	/* 
	 * This method is responsible for changing the renderer; its called when you click
	 * either the OpenGL or Software button in the settings frame; indicates what renderer
	 * we want to use and shows the main loop that a change of the renderer is requested
	 */
	public static void setSoftRenderer(boolean softRend) {
		
		// Use the software renderer?
		softRenderer = softRend;
		
		// Request a renderer change
		switchRenderer = true;
	}
	
	// Sets the exit variable so we can leave the main loop
	public static void setExit() {
		exit = true;
	}
	
	// Get the position of our main frame
	public static Point getPosition() {
		return mainFrame.getLocation();
	}
	
	// Get the size of our frame
	public static Dimension getSize() {
		return mainFrame.getSize();
	}
	
	// Main method
	public static void main(String[] args) {
		JPCTTutorial3 helloWorld = new JPCTTutorial3();
	}
}
