import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Point;

import com.threed.jpct.Camera;
import com.threed.jpct.FrameBuffer;
import com.threed.jpct.IRenderer;
import com.threed.jpct.Lights;
import com.threed.jpct.Object3D;
import com.threed.jpct.Primitives;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.World;

/*
 * Created on 30.11.2003
 * 
 * This is the first part of the jPCT Tutorial. Here you will learn the basics of creating 
 * applications with jPCT. For the beginning you can see that every application consists 
 * of three parts: the initializing of the world, the creation of a container for our 
 * program and the main loop. Later we will extend our simple recipe.
 * 
 * This tutorial creates just a cube positioned at the origin of the coord system. In 
 * this tutorial we will use the software renderer so you don't need hardware acceleration 
 * to see the reasults.
 */

/*
 * Our main class starts here
 */
public class KeyBoardTest {

	
	// Some jPCT related lines. 
	private Object3D cube = null;
	private World theWorld = null;
	private Camera camera = null;
	private FrameBuffer buffer = null;
		
	/* 
	 * The width and the height of our framebuffer. Remember, this is _not_ the dimension
	 * of our Frame.
	 */	
	private static int width = 640;
	private static int height = 480;
	
	// Some lines for declaring AWT stuff
	private static Frame mainFrame = null;
	private Graphics frameGraphics = null;
	private static int titleBarHeight = 0;
	private static int leftBorderWidth = 0;
	
	private boolean exit = false;
	private boolean softRenderer = true;
	private boolean switchRenderer = false;
		
	TestWindow test = null;
	static TestWrapper testw = null;
	
	// The constructor
	public KeyBoardTest() {
		// First we create our empty world.
		theWorld = new World();
		testw = new TestWrapper();
		/*
		 * Deactivate the overbright lighting, so our software renderer looks similar
		 * to the OpenGL renderer.	
		 */
		theWorld.getLights().setOverbrightLighting(Lights.OVERBRIGHT_LIGHTING_DISABLED);
		
		/*
		 * We want our scene to be lit by an ambient light source. With the parameters
		 * we control the intesity of the light so our positive params will light up
		 * the scene. With high values we can see our cube without any other light sources,
		 * but this won't look very realistic because every face reflects exactly the
		 * same amount of light so we won't see any edges.
		 */
		theWorld.setAmbientLight(20, 20, 20);
		
		/*
		 * Create a light and add it to our world. The first param ist the position of
		 * the light, the second one the color.
		 */		
	 	theWorld.addLight(new SimpleVector(10.0f, -10.0f, -30.0f), Color.red);
		
		// Here we create our cube with a scaling of 10 and add it to our world.				
		cube = Primitives.getCube(10.0f);
		theWorld.addObject(cube);
		
		/* 
		 * Get the camera from our world and move it 60 units "backwards" (in negative
		 * z-direction) so that we can see our cube.
		 */ 
		camera = theWorld.getCamera();
		camera.moveCamera(Camera.CAMERA_MOVEOUT, 60);
		
		// Now we have to build our objects.
		theWorld.buildAllObjects();
		
		// Create a frame.
		initializeFrame();
		test = new TestWindow(this);
				
		// Start our main loop.
		gameloop();
	}
	
	// Here we create our frame and get its graphics context.
	private void initializeFrame() {
		// Create the frame, give it a title and pack everything.
		mainFrame = new Frame();
		mainFrame.setTitle("jPCT Tutorial #1: Hello World");
		mainFrame.pack();
		
		/* 
		 * Set the size of our frame. The size is calculated from the width and the height
		 * of the framebuffer and the frames borderwidth and -height (incl. title).
		 */		
		Insets insets = mainFrame.getInsets();
		titleBarHeight=insets.top;
		leftBorderWidth=insets.left;
		mainFrame.setSize(width+leftBorderWidth+insets.right, height+titleBarHeight+insets.bottom);
		mainFrame.addKeyListener(testw);
		
		// We don't want our frame to be resizeable.
		mainFrame.setResizable(false);
		
		// Show the frame.
		mainFrame.show();
		
		// Get our graphics context so that we can work with it.	
		frameGraphics = mainFrame.getGraphics();
		
		// Add a window listener (for closing)
	}			
	
	// The main loop of the application.
	private void gameloop() {
		/*
		 * Create our framebuffer with the given width and height. We don't 
		 * want to use a special sampling mode (we don't want to seperate our buffer
		 * grid into smaller units; no up- or downsampling). 
		 */
		buffer = new FrameBuffer(width, height, FrameBuffer.SAMPLINGMODE_NORMAL);
		buffer.enableRenderer(IRenderer.RENDERER_SOFTWARE, IRenderer.MODE_OPENGL);		
		
		// Optimize the buffer.
		buffer.optimizeBufferAccess();
		
		/* 
		 * The main loop. Calculate the scene, write it to the buffer, swap the buffers 
		 * and draw the content of current buffer onto the frame using the graphics context.
		 */ 
		while (!exit) {
			testw.keybPoll();
			if (switchRenderer)
				changeRenderer();
			if (testw.keyUP())
				camera.moveCamera(Camera.CAMERA_MOVEIN, 0.1f);
			if (testw.keyDOWN())
				camera.moveCamera(Camera.CAMERA_MOVEOUT, 0.1f);	
			if (testw.keyLEFT())
				camera.rotateCameraY(0.01f);	
			if (testw.keyRIGHT())
				camera.rotateCameraY(-0.01f);										
			buffer.clear();
			theWorld.renderScene(buffer);
			theWorld.draw(buffer);
			buffer.update();
			if (!softRenderer)
				buffer.displayGLOnly();
			else
				buffer.display(frameGraphics, leftBorderWidth, titleBarHeight);
		}
		System.exit(0);
	}
	
	public static Point getPosition() {
		return mainFrame.getLocation();
	}
	
	public static Dimension getSize() {
		return mainFrame.getSize();
	}
	
	// Very complex stuff:)
	public static void main(String[] args) {
		KeyBoardTest helloWorld = new KeyBoardTest();
	}
	
	public void changeRenderer() {
		if (!softRenderer) {
			mainFrame.hide();
			buffer.enableRenderer(IRenderer.RENDERER_OPENGL, IRenderer.MODE_OPENGL);
			buffer.disableRenderer(IRenderer.RENDERER_SOFTWARE);
			testw.setSoftRenderer(false);
			testw.createKeyboard();
			softRenderer = false;
			switchRenderer = false;
		}
		else {
			buffer.disableRenderer(IRenderer.RENDERER_OPENGL);
			buffer.enableRenderer(IRenderer.RENDERER_SOFTWARE, IRenderer.MODE_OPENGL);
			testw.setSoftRenderer(true);
			testw.destroyKeyboard();
			mainFrame.hide();
			mainFrame.dispose();
			softRenderer = true;	
			initializeFrame();
			switchRenderer = false;
		}	
	}
	
	public void setSoftRenderer(boolean softRenderer) {
		this.softRenderer = softRenderer;
		switchRenderer = true;
	}
	
	public void setExit() {
		this.exit = true;
	}
}
